@@ -24,8 +24,8 @@ instruction:
 from __future__ import annotations
 
 import re
-from collections import ChainMap
-from typing import TYPE_CHECKING, Any, MutableSequence
+from typing import TYPE_CHECKING, Any
+from warnings import warn
 from xml.etree.ElementTree import Element
 
 import yaml
@@ -39,9 +39,10 @@ from mkdocstrings.handlers.base import BaseHandler, CollectionError, CollectorIt
 from mkdocstrings.loggers import get_logger
 
 if TYPE_CHECKING:
+    from collections.abc import MutableSequence
+
     from markdown import Markdown
-    from markdown.blockparser import BlockParser
-    from mkdocs_autorefs.plugin import AutorefsPlugin
+    from mkdocs_autorefs import AutorefsPlugin
 
 
 log = get_logger(__name__)
@@ -61,24 +62,20 @@ class AutoDocProcessor(BlockProcessor):
 
     def __init__(
         self,
-        parser: BlockParser,
         md: Markdown,
-        config: dict,
+        *,
         handlers: Handlers,
         autorefs: AutorefsPlugin,
     ) -> None:
         """Initialize the object.
 
         Arguments:
-            parser: A `markdown.blockparser.BlockParser` instance.
             md: A `markdown.Markdown` instance.
-            config: The [configuration][mkdocstrings.plugin.PluginConfig] of the `mkdocstrings` plugin.
             handlers: The handlers container.
             autorefs: The autorefs plugin instance.
         """
-        super().__init__(parser=parser)
+        super().__init__(parser=md.parser)
         self.md = md
-        self._config = config
         self._handlers = handlers
         self._autorefs = autorefs
         self._updated_envs: set = set()
@@ -123,45 +120,15 @@ class AutoDocProcessor(BlockProcessor):
         if match:
             identifier = match["name"]
             heading_level = match["heading"].count("#")
-            log.debug(f"Matched '::: {identifier}'")
+            log.debug("Matched '::: %s'", identifier)
 
             html, handler, data = self._process_block(identifier, block, heading_level)
             el = Element("div", {"class": "mkdocstrings"})
             # The final HTML is inserted as opaque to subsequent processing, and only revealed at the end.
             el.text = self.md.htmlStash.store(html)
-            # So we need to duplicate the headings directly (and delete later), just so 'toc' can pick them up.
-            headings = handler.get_headings()
-            el.extend(headings)
 
-            page = self._autorefs.current_page
-            if page is not None:
-                for heading in headings:
-                    rendered_anchor = heading.attrib["id"]
-                    self._autorefs.register_anchor(page, rendered_anchor)
-
-                    if "data-role" in heading.attrib:
-                        self._handlers.inventory.register(
-                            name=rendered_anchor,
-                            domain=handler.domain,
-                            role=heading.attrib["data-role"],
-                            priority=1,  # register with standard priority
-                            uri=f"{page}#{rendered_anchor}",
-                        )
-
-                        # also register other anchors for this object in the inventory
-                        try:
-                            data_object = handler.collect(rendered_anchor, handler.fallback_config)
-                        except CollectionError:
-                            continue
-                        for anchor in handler.get_anchors(data_object):
-                            if anchor not in self._handlers.inventory:
-                                self._handlers.inventory.register(
-                                    name=anchor,
-                                    domain=handler.domain,
-                                    role=heading.attrib["data-role"],
-                                    priority=2,  # register with lower priority
-                                    uri=f"{page}#{rendered_anchor}",
-                                )
+            if handler.outer_layer:
+                self._process_headings(handler, el)
 
             parent.append(el)
 
@@ -191,46 +158,136 @@ class AutoDocProcessor(BlockProcessor):
         Returns:
             Rendered HTML, the handler that was used, and the collected item.
         """
-        config = yaml.safe_load(yaml_block) or {}
-        handler_name = self._handlers.get_handler_name(config)
+        local_config = yaml.safe_load(yaml_block) or {}
+        handler_name = self._handlers.get_handler_name(local_config)
 
-        log.debug(f"Using handler '{handler_name}'")
-        handler_config = self._handlers.get_handler_config(handler_name)
-        handler = self._handlers.get_handler(handler_name, handler_config)
-
-        global_options = handler_config.get("options", {})
-        local_options = config.get("options", {})
-        options = ChainMap(local_options, global_options)
+        log.debug("Using handler '%s'", handler_name)
+        handler = self._handlers.get_handler(handler_name)
 
+        local_options = local_config.get("options", {})
         if heading_level:
-            options = ChainMap(options, {"heading_level": heading_level})  # like setdefault
+            # Heading level obtained from Markdown (`##`) takes precedence.
+            local_options["heading_level"] = heading_level
+
+        # YORE: Bump 1: Replace block with line 2.
+        if handler.get_options.__func__ is not BaseHandler.get_options:  # type: ignore[attr-defined]
+            options = handler.get_options(local_options)
+        else:
+            warn(
+                "mkdocstrings v1 will start using your handler's `get_options` method to build options "
+                "instead of merging the global and local options (dictionaries). ",
+                DeprecationWarning,
+                stacklevel=1,
+            )
+            handler_config = self._handlers.get_handler_config(handler_name)
+            global_options = handler_config.get("options", {})
+            options = {**global_options, **local_options}
 
         log.debug("Collecting data")
         try:
             data: CollectorItem = handler.collect(identifier, options)
         except CollectionError as exception:
-            log.error(str(exception))  # noqa: TRY400
-            if PluginError is SystemExit:  # TODO: when MkDocs 1.2 is sufficiently common, this can be dropped.
-                log.error(f"Error reading page '{self._autorefs.current_page}':")  # noqa: TRY400
+            log.error("%s", exception)  # noqa: TRY400
             raise PluginError(f"Could not collect '{identifier}'") from exception
 
         if handler_name not in self._updated_envs:  # We haven't seen this handler before on this document.
             log.debug("Updating handler's rendering env")
-            handler._update_env(self.md, self._config)
+            handler._update_env(self.md, config=self._handlers._tool_config)
             self._updated_envs.add(handler_name)
 
         log.debug("Rendering templates")
         try:
             rendered = handler.render(data, options)
         except TemplateNotFound as exc:
-            theme_name = self._config["theme_name"]
             log.error(  # noqa: TRY400
-                f"Template '{exc.name}' not found for '{handler_name}' handler and theme '{theme_name}'.",
+                "Template '%s' not found for '%s' handler and theme '%s'.",
+                exc.name,
+                handler_name,
+                self._handlers._theme,
             )
             raise
 
         return rendered, handler, data
 
+    def _process_headings(self, handler: BaseHandler, element: Element) -> None:
+        # We're in the outer handler layer, as well as the outer extension layer.
+        #
+        # The "handler layer" tracks the nesting of the autodoc blocks, which can appear in docstrings.
+        #
+        # - Render ::: Object1                  # Outer handler layer
+        #   - Render Object1's docstring        # Outer handler layer
+        #     - Docstring renders ::: Object2   # Inner handler layers
+        #       - etc.                          # Inner handler layers
+        #
+        # The "extension layer" tracks whether we're converting an autodoc instruction
+        # or nested content within it, like docstrings. Markdown conversion within Markdown conversion.
+        #
+        # - Render ::: Object1                  # Outer extension layer
+        #   - Render Object1's docstring        # Inner extension layer
+        #
+        # The generated HTML was just stashed, and the `toc` extension won't be able to see headings.
+        # We need to duplicate the headings directly, just so `toc` can pick them up,
+        # otherwise they wouldn't appear in the final table of contents.
+        #
+        # These headings are generated by the `BaseHandler.do_heading` method (Jinja filter),
+        # which runs in the inner extension layer, and not in the outer one where we are now.
+        headings = handler.get_headings()
+        element.extend(headings)
+        # These duplicated headings will later be removed by our `_HeadingsPostProcessor` processor,
+        # which runs right after `toc` (see `MkdocstringsExtension.extendMarkdown`).
+        #
+        # If we were in an inner handler layer, we wouldn't do any of this
+        # and would just let headings bubble up to the outer handler layer.
+
+        page = self._autorefs.current_page
+        if page is not None:
+            for heading in headings:
+                rendered_id = heading.attrib["id"]
+                self._autorefs.register_anchor(page, rendered_id, primary=True)
+
+                # Register all identifiers for this object
+                # both in the autorefs plugin and in the inventory.
+                aliases: tuple[str, ...]
+                # YORE: Bump 1: Replace block with line 16.
+                if hasattr(handler, "get_anchors"):
+                    warn(
+                        "The `get_anchors` method is deprecated. "
+                        "Declare a `get_aliases` method instead, accepting a string (identifier) "
+                        "instead of a collected object.",
+                        DeprecationWarning,
+                        stacklevel=1,
+                    )
+                    try:
+                        data_object = handler.collect(rendered_id, getattr(handler, "fallback_config", {}))
+                    except CollectionError:
+                        aliases = ()
+                    else:
+                        aliases = handler.get_anchors(data_object)
+                else:
+                    aliases = handler.get_aliases(rendered_id)
+
+                for alias in aliases:
+                    if alias != rendered_id:
+                        self._autorefs.register_anchor(page, alias, rendered_id, primary=False)
+
+                if "data-role" in heading.attrib:
+                    self._handlers.inventory.register(
+                        name=rendered_id,
+                        domain=handler.domain,
+                        role=heading.attrib["data-role"],
+                        priority=1,  # Register with standard priority.
+                        uri=f"{page}#{rendered_id}",
+                    )
+                    for alias in aliases:
+                        if alias not in self._handlers.inventory:
+                            self._handlers.inventory.register(
+                                name=alias,
+                                domain=handler.domain,
+                                role=heading.attrib["data-role"],
+                                priority=2,  # Register with lower priority.
+                                uri=f"{page}#{rendered_id}",
+                            )
+
 
 class _HeadingsPostProcessor(Treeprocessor):
     def run(self, root: Element) -> None:
@@ -270,18 +327,15 @@ class MkdocstringsExtension(Extension):
     It cannot work outside of `mkdocstrings`.
     """
 
-    def __init__(self, config: dict, handlers: Handlers, autorefs: AutorefsPlugin, **kwargs: Any) -> None:
+    def __init__(self, handlers: Handlers, autorefs: AutorefsPlugin, **kwargs: Any) -> None:
         """Initialize the object.
 
         Arguments:
-            config: The configuration items from `mkdocs` and `mkdocstrings` that must be passed to the block processor
-                when instantiated in [`extendMarkdown`][mkdocstrings.extension.MkdocstringsExtension.extendMarkdown].
             handlers: The handlers container.
             autorefs: The autorefs plugin instance.
             **kwargs: Keyword arguments used by `markdown.extensions.Extension`.
         """
         super().__init__(**kwargs)
-        self._config = config
         self._handlers = handlers
         self._autorefs = autorefs
 
@@ -294,7 +348,7 @@ class MkdocstringsExtension(Extension):
             md: A `markdown.Markdown` instance.
         """
         md.parser.blockprocessors.register(
-            AutoDocProcessor(md.parser, md, self._config, self._handlers, self._autorefs),
+            AutoDocProcessor(md, handlers=self._handlers, autorefs=self._autorefs),
             "mkdocstrings",
             priority=75,  # Right before markdown.blockprocessors.HashHeaderProcessor
         )
